import numpy as np
from scipy.optimize import linear_sum_assignment
import torch
import torch.nn as nn
import torch.nn.functional as F
from sklearn.cluster import KMeans
from sklearn.manifold import spectral_embedding
from sklearn.neighbors import kneighbors_graph
from sklearn.model_selection import train_test_split
from sklearn.metrics import adjusted_mutual_info_score, adjusted_rand_score
import random
import pickle
import os


class Four_layer_FNN(nn.Module):
    """The model used to learn the embedding.

    Parameters
    ----------
    n_feature : int
        The number of input features.
    n_hidden1 : int
        The number of neurons in the first hidden layer.
    n_hidden2 : int
        The number of neurons in the second hidden layer.
    n_hidden3 : int
        The number of neurons in the third hidden layer.
    n_output : int
        The number of output features.

    Attributes
    ----------
    hidden1 : torch.nn.Linear
        The first hidden layer.
    hidden2 : torch.nn.Linear
        The second hidden layer.
    hidden3 : torch.nn.Linear
        The third hidden layer.
    predict : torch.nn.Linear
        The output layer.

    Examples
    --------
    >>> model = Four_layer_FNN(64, 128, 256, 64, 10)
    >>> model
    Four_layer_FNN(
        (hidden1): Linear(in_features=64, out_features=128, bias=True)
        (hidden2): Linear(in_features=128, out_features=256, bias=True)
        (hidden3): Linear(in_features=256, out_features=64, bias=True)
        (predict): Linear(in_features=64, out_features=10, bias=True)
    )
    """

    def __init__(self, n_feature, n_hidden1, n_hidden2, n_hidden3, n_output):
        super().__init__()
        self.hidden1 = nn.Linear(n_feature, n_hidden1)
        self.hidden2 = nn.Linear(n_hidden1, n_hidden2)
        self.hidden3 = nn.Linear(n_hidden2, n_hidden3)
        self.predict = nn.Linear(n_hidden3, n_output)

    def forward(self, x):
        """Forward propagation.

        Parameters
        ----------
        x : torch.Tensor
            The input tensor.

        Returns
        -------
        x : torch.Tensor
            The output tensor.
        """
        x = F.relu(self.hidden1(x))
        x = F.relu(self.hidden2(x))
        x = F.relu(self.hidden3(x))
        x = self.predict(x)
        return x


class Accuracy:
    """Calculate the accuracy of clustering.

    Parameters
    ----------
    y_true : list
        True labels.
    y_pred : list
        Predicted labels.

    Attributes
    ----------
    y_true : list
        True labels.
    y_pred : list
        Predicted labels.

    Examples
    --------
    >>> from ParametricSpectralClustering import Accuracy
    >>> from sklearn.datasets import load_digits
    >>> from sklearn.cluster import KMeans
    >>> digits = load_digits()
    >>> X = digits.data/16
    >>> y = digits.target
    >>> y_pred = KMeans(n_clusters=10, random_state=0).fit_predict(X)
    >>> acc = Accuracy(y, y_pred)
    >>> acc.acc_report()
    Clustering Accuracy: 0.7935447968836951
    Adjusted rand index: 0.670943009820327
    Adjusted mutual information: 0.7481788599584174
    """

    def __init__(self, y_true, y_pred):
        self.y_true = y_true
        self.y_pred = y_pred

    def cluster_acc(self):
        """Calculate the clustering accuracy.

        Parameters
        ----------
        self : object
            The instance itself.

        Returns
        -------
        acc : float
            The clustering accuracy.
        """
        self.y_true = self.y_true.astype(np.int64)
        assert self.y_pred.size == self.y_true.size
        D = max(self.y_pred.max(), self.y_true.max()) + 1
        w = np.zeros((D, D), dtype=np.int64)
        for i in range(self.y_pred.size):
            w[self.y_pred[i], self.y_true[i]] += 1
        row_ind, col_ind = linear_sum_assignment(w.max() - w)
        return w[row_ind, col_ind].sum() * 1.0 / self.y_pred.size

    def ARI(self):
        """Calculate the adjusted rand index.

        Parameters
        ----------
        self : object
            The instance itself.

        Returns
        -------
        ari : float
            The adjusted rand index.
        """
        return adjusted_rand_score(self.y_true, self.y_pred)

    def AMI(self):
        """Calculate the adjusted mutual information.

        Parameters
        ----------
        self : object
            The instance itself.

        Returns
        -------
        ami : float
            The adjusted mutual information.
        """
        return adjusted_mutual_info_score(self.y_true, self.y_pred)

    def acc_report(self):
        """Report the accuracy of clustering.

        Parameters
        ----------
        self : object
            The instance itself.

        Returns
        -------
        clusterAcc : float
            The clustering accuracy.
        ari : float
            The adjusted rand index.
        ami : float
            The adjusted mutual information.
        """
        clusterAcc = self.cluster_acc()
        ari = self.ARI()
        ami = self.AMI()
        return clusterAcc, ari, ami


class PSC:
    """Parametric Spectral Clustering.

    Parameters
    ----------
    n_neighbor : int, default=8
        Number of neighbors to use when constructing the adjacency matrix using k-nearest neighbors.
    n_clusters : int, default=10
        Number of clusters.
    model : torch.nn.Module
        The model used to learn the embedding.
    criterion : torch.nn.modules.loss, default=nn.MSELoss()
        The loss function used to train the model.
    epochs : int, default=50
        Number of epochs to train the model.
    sampling_ratio : float, default=0.3
        The spliting rate of the testing data.
    batch_size_data : int, default=50
        The batch size of the training data.
    batch_size_dataloader : int, default=20
        The batch size of the dataloader.
    clustering_method : sklearn.cluster, default=None
        The clustering method used to cluster the embedding.
    n_components : int, default=0
        The number of embedding dimensions.
    random_state : int, default=None
        The random state.

    Attributes
    ----------
    n_neighbor : int
        Number of neighbors to use when constructing the adjacency matrix using k-nearest neighbors.
    n_clusters : int
        Number of clusters.
    model : torch.nn.Module
        The model used to learn the embedding.
    criterion : torch.nn.modules.loss
        The loss function used to train the model.
    sampling_ratio : float
        The spliting rate of the testing data.
    optimizer : torch.optim
        The optimizer used to train the model.
    clustering : str
        The clustering method used to cluster the embedding.
    n_components : int
        The number of embedding dimensions.
    random_state : int
        The random state.
    epochs : int
        Number of epochs to train the model.
    model_fitted : bool
        Whether the model has been fitted.
    batch_size_data : int
        The batch size of the training data.
    batch_size_dataloader : int
        The batch size of the dataloader.
    dataloader : torch.utils.data.DataLoader
        The dataloader used to train the model.


    Examples
    --------
    >>> from ParametricSpectralClustering import PSC, Four_layer_FNN
    >>> from sklearn.datasets import load_digits
    >>> from sklearn.cluster import KMeans
    >>> digits = load_digits()
    >>> X = digits.data/16
    >>> cluster_method = KMeans(n_clusters=10, init="k-means++", n_init=1, max_iter=100, algorithm='elkan')
    >>> model = Four_layer_FNN(64, 128, 256, 64, 10)
    >>> psc = PSC(model=model, clustering_method=cluster_method, n_neighbor=10, sampling_ratio=0, batch_size_data=1797)
    >>> psc.fit(X)
    >>> psc.save_model("model")
    >>> cluster_idx = psc.predict(X)

    >>> from ParametricSpectralClustering import PSC, Four_layer_FNN
    >>> from sklearn.datasets import load_digits
    >>> from sklearn.cluster import KMeans
    >>> digits = load_digits()
    >>> X = digits.data/16
    >>> cluster_method = KMeans(n_clusters=10, init="k-means++", n_init=1, max_iter=100, algorithm='elkan')
    >>> model = Four_layer_FNN(64, 128, 256, 64, 10)
    >>> psc = PSC(model=model, clustering_method=cluster_method)
    >>> psc.load_model("model")
    >>> cluster_idx = psc.predict(X)
    """

    def __init__(
        self,
        n_neighbor=8,
        n_clusters=10,
        model=Four_layer_FNN(64, 128, 256, 64, 10),
        criterion=nn.MSELoss(),
        epochs=50,
        sampling_ratio=0.3,
        batch_size_data=50,
        batch_size_dataloader=20,
        clustering_method=None,
        n_components=0,
        random_state=None,
    ) -> None:
        self.n_neighbor = n_neighbor
        self.n_clusters = n_clusters
        self.model = model
        self.criterion = criterion
        self.sampling_ratio = sampling_ratio
        self.optimizer = torch.optim.Adam(model.parameters(), lr=0.001)

        if random_state is None:
            self.random_state = random.randint(1, 100)
        else:
            self.random_state = random_state

        if clustering_method is None:
            self.clustering = KMeans(
                n_clusters=self.n_clusters,
                init="k-means++",
                n_init=1,
                max_iter=100,
                algorithm="elkan",
                random_state=self.random_state,
            )
        else:
            self.clustering = clustering_method

        if n_components == 0:
            self.n_components = self.n_clusters
        else:
            self.n_components = n_components

        self.epochs = epochs
        self.model_fitted = False

        self.batch_size_data = batch_size_data
        self.batch_size_dataloader = batch_size_dataloader

    def __loss_calculation(self):
        running_loss = 0.0

        for inputs, labels in self.dataloader:
            self.optimizer.zero_grad()
            outputs = self.model(inputs)
            loss = self.criterion(outputs, labels)
            loss.backward()
            self.optimizer.step()
            running_loss += loss.item()

        return running_loss / len(self.dataloader)

    def __train_model(self, X, x):
        self.model_fitted = True
        connectivity = kneighbors_graph(
            X, n_neighbors=self.n_neighbor, include_self=False
        )
        affinity_matrix_ = 0.5 * (connectivity + connectivity.T)
        embedding = spectral_embedding(
            affinity_matrix_,
            n_components=self.n_components,
            eigen_solver="arpack",
            random_state=1,
            eigen_tol="auto",
            drop_first=False,
        )
        u = torch.from_numpy(embedding).type(torch.FloatTensor)
        dataset = torch.utils.data.TensorDataset(x, u)
        # not sure whether shuffle = True will affect the result
        dataloader = torch.utils.data.DataLoader(
            dataset, batch_size=self.batch_size_dataloader, shuffle=False
        )
        self.dataloader = dataloader
        total_loss = 0
        for _ in range(self.epochs):
            loss = self.__loss_calculation()
            total_loss += loss
        return total_loss / self.epochs

    def __check_file_exist(self, file_name) -> bool:
        return os.path.exists(file_name)

    def __check_clustering_method(self) -> None:
        if self.clustering is None:
            raise ValueError("No clustering method assigned.")

    def __check_model(self) -> None:
        if self.model is None:
            raise ValueError("No model assigned.")

    def fit(self, X):
        """Train a model to convert input features into spectral embeddings.

        Parameters
        ----------
        X : array-like of shape
            Training data.

        Returns
        -------
        No return value
        """

        self.__check_clustering_method()
        self.__check_model()

        x = torch.from_numpy(X).type(torch.FloatTensor)

        if self.sampling_ratio >= 1 or self.sampling_ratio < 0:
            raise AttributeError(
                f"'test_spliting_rate' should be not less than 0 and less than 1."
            )

        if self.sampling_ratio == 0:
            X_train, x_train = X, x

        else:
            X_train, _, x_train, _ = train_test_split(
                X,
                x,
                test_size=self.sampling_ratio,
                random_state=self.random_state,
            )

        batch_size = self.batch_size_data
        total_loss = 0
        i = 1
        # Train the model in mini-batches
        for start_idx in range(0, len(X_train), batch_size):
            end_idx = start_idx + batch_size
            X_batch = X_train[start_idx:end_idx]
            x_batch = x_train[start_idx:end_idx]
            loss = self.__train_model(X_batch, x_batch)
            total_loss += loss
            if i % 20 == 0:
                total_loss = 0
            i += 1

    def fit_predict(self, X):
        """Fit the model and predict the closest cluster each sample in X belongs to.

        Parameters
        ----------
        X : array-like of shape (n_samples, n_features)
            Training data.

        Returns
        -------
        cluster_index : array-like of shape (n_samples,)
            Index of the cluster each sample belongs to.
        """
        self.fit(X)

        if hasattr(self.clustering, "fit_predict") is False:
            raise AttributeError(
                f"'{type(self.clustering)}' object has no attribute 'fit_predict'"
            )

        return self.predict(X)

    def predict(self, X):
        """Predict the closest cluster each sample in X belongs to.

        Parameters
        ----------
        X : array-like of shape (n_samples, n_features)
            New data.

        Returns
        -------
        cluster_index : array-like of shape (n_samples,)
            Index of the cluster each sample belongs to.
        """

        x = torch.from_numpy(X).type(torch.FloatTensor)

        # turn input data points into low-dim embedding
        emb = self.model(x).detach().numpy()

        if hasattr(self.clustering, "fit_predict") is False:
            raise AttributeError(
                f"'{type(self.clustering)}' object has no attribute 'predict'"
            )

        return self.clustering.fit_predict(emb)

    def save_model(self, path: str) -> None:
        """Save the model to a file.

        Parameters
        ----------
        path : str
            The path of the file.
        """

        with open(path, "wb") as f:
            pickle.dump(self.model, f)

    def load_model(self, path: str) -> None:
        """Load the model from a file.

        Parameters
        ----------
        path : str
            The path of the file.
        """
        if self.__check_file_exist(path) is False:
            raise FileNotFoundError(f"No such file or directory: '{path}'")

        with open(path, "rb") as f:
            self.model = pickle.load(f)



import unittest
from ParametricSpectralClustering import PSC, Four_layer_FNN
from sklearn.cluster import KMeans
from sklearn.datasets import load_digits
import numpy as np
import torch
import random
import pickle
import os

digits = load_digits()
x = digits.data / 16
y = digits.target
torch.manual_seed(0)
np.random.seed(0)
random.seed(0)
clust_method = KMeans(
    n_clusters=10, init="k-means++", n_init=1, max_iter=100, algorithm="elkan"
)
model = Four_layer_FNN(64, 128, 256, 64, 10)
psc = PSC(
    model=model,
    clustering_method=clust_method,
    sampling_ratio=0.3,
    batch_size_data=1797,
)
cluster_idx = psc.fit_predict(x)


class testPSC(unittest.TestCase):
    def test_init(self):
        self.assertIs(type(psc.n_neighbor), int)
        self.assertIs(type(psc.n_clusters), int)
        self.assertIs(type(psc.model), Four_layer_FNN)
        self.assertIs(type(psc.criterion), torch.nn.modules.loss.MSELoss)
        self.assertIs(type(psc.sampling_ratio), float)
        self.assertIs(type(psc.optimizer), torch.optim.Adam)
        self.assertIs(type(psc.clustering), KMeans)
        self.assertIs(type(psc.n_components), int)
        self.assertIs(type(psc.random_state), int)
        self.assertIs(type(psc.epochs), int)
        self.assertIs(type(psc.model_fitted), bool)
        self.assertIs(type(psc.batch_size_data), int)
        self.assertIs(type(psc.batch_size_dataloader), int)
        self.assertIs(type(psc.dataloader), torch.utils.data.dataloader.DataLoader)

        self.assertEqual(psc.n_neighbor, 8)
        self.assertEqual(psc.n_clusters, 10)
        self.assertEqual(psc.model, model)
        self.assertEqual(psc.epochs, 50)
        self.assertEqual(psc.sampling_ratio, 0.3)
        self.assertEqual(psc.batch_size_data, 1797)
        self.assertEqual(psc.clustering, clust_method)

    # output shape of fit_predict and predict
    def test_output_shape(self):
        self.assertEqual(x.shape, (1797, 64))
        output = psc.fit_predict(x)
        self.assertEqual(output.shape, (1797,))
        output = psc.predict(x)
        self.assertEqual(output.shape, (1797,))

    def test_save_model(self):
        psc.save_model("test_save_model")
        self.assertEqual(os.path.exists("test_save_model"), True)

        psc.fit(x)
        psc.save_model("test_save_fit_model")
        self.assertEqual(os.path.exists("test_save_fit_model"), True)

        m1 = None
        m2 = None
        with open("test_save_model", "wb") as f1:
            pickle.dump(m1, f1)
        with open("test_save_fit_model", "wb") as f2:
            pickle.dump(m2, f2)
        self.assertEqual(m1, m2)

    def test_load_model(self):
        psc.load_model("test_load_model")
        self.assertEqual(os.path.exists("test_load_model"), True)
        self.assertEqual(psc.model_fitted, True)


if __name__ == "__main__":
    unittest.main()



import unittest
from ParametricSpectralClustering import PSC, Accuracy
from sklearn.datasets import load_digits
import numpy as np
from unittest.mock import patch
import io

digits = load_digits()
x = digits.data / 16
y = digits.target
psc = PSC(batch_size_data=1797)
cluster_idx = psc.fit_predict(x)
acc = Accuracy(y_true=y, y_pred=cluster_idx)


class testAccuracy(unittest.TestCase):
    def test_cluster_acc(self):
        self.assertIs(type(acc.cluster_acc()), np.float64)

    def test_ARI(self):
        self.assertIs(type(acc.ARI()), float)

    def test_AMI(self):
        self.assertIs(type(acc.AMI()), np.float64)

    def test_acc_report(self):
        with patch("sys.stdout", new_callable=io.StringIO) as mock_stdout:
            acc.acc_report()
            printed_output = mock_stdout.getvalue()
        self.assertTrue(printed_output.strip(), msg="Function didn't print anything")


if __name__ == "__main__":
    unittest.main()



import unittest
from ParametricSpectralClustering import Four_layer_FNN
import torch
import numpy as np

m = Four_layer_FNN(64, 128, 256, 64, 10)


class test_Four_layer_FNN(unittest.TestCase):
    def test_init(self):
        self.assertIs(type(m.hidden1), torch.nn.Linear)
        self.assertIs(type(m.hidden2), torch.nn.Linear)
        self.assertIs(type(m.hidden3), torch.nn.Linear)
        self.assertIs(type(m.predict), torch.nn.Linear)

    @torch.no_grad()
    def test_output_shape(self):
        x = torch.randn(1797, 64)
        outputs = m(x)
        self.assertEqual(10, outputs.shape[1])


if __name__ == "__main__":
    unittest.main()

# python -m unittest -v test_Four_layer_FNN.test_Four_layer_FNN



import numpy as np
import sys
import argparse
import pandas as pd
from sklearn.cluster import KMeans
from ParametricSpectralClustering import PSC

parser = argparse.ArgumentParser()
parser.add_argument("-data", "--train_data", type=str, help="Training data")
parser.add_argument(
    "-n_cluster", "--n_cluster", type=int, default=10, help="Number of clusters"
)
parser.add_argument("-path", "--model_path", type=str, help="The pretrained model path")
parser.add_argument(
    "-cluster_result_format",
    "--cluster_result_format",
    type=str,
    default="csv",
    help="Save the result in csv format or txt format",
)
args = parser.parse_args()


def __check_args():
    if args.train_data is not None and args.train_data[-3:] not in [
        "npy",
        "csv",
        "txt",
    ]:
        raise ValueError("The training data must be in .npy, .csv or .txt format.")
    if args.n_cluster is not None and args.n_cluster <= 0:
        raise ValueError("n_cluster must be integer and greater than 0.")
    if args.model_path is None:
        raise ValueError("Model path not assigned.")
    if args.cluster_result_format is not None and args.cluster_result_format not in [
        "csv",
        "txt",
    ]:
        raise ValueError("The saving format must be .csv or .txt.")


def __load_data():
    if args.train_data[-3:] == "npy":
        # load from npy
        x = np.load(args.train_data)
    elif args.train_data[-3:] == "csv":
        # load from csv
        df = pd.read_csv(args.train_data, header=None)
        x = df.to_numpy()
    elif args.train_data[-3:] == "txt":
        # load from txt
        x = np.loadtxt(args.train_data, dtype=int)

    return x


def main(argv):
    __check_args()

    x = __load_data()

    cluster_method = KMeans(
        n_clusters=args.n_cluster,
        init="k-means++",
        n_init=1,
        max_iter=100,
        algorithm="elkan",
    )
    psc = PSC(clustering_method=cluster_method, test_splitting_rate=0)
    psc.load_model(args.model_path)
    cluster_idx = psc.predict(x)

    if args.cluster_result_format == "csv":
        df = pd.DataFrame(cluster_idx)
        df.to_csv(
            args.train_data[:-4] + "_cluster_result.csv", index=False, header=False
        )
    elif args.cluster_result_format == "txt":
        f = open(args.train_data[:-4] + "_cluster_result.txt", "w")
        np.set_printoptions(threshold=sys.maxsize)
        # print(cluster_idx)
        f.write(str(cluster_idx) + " ")
        f.close()

    print("Finished")


if __name__ == "__main__":
    main(sys.argv)



import sys
import argparse
import pandas as pd
import numpy as np
from sklearn.cluster import KMeans
from ParametricSpectralClustering import PSC

parser = argparse.ArgumentParser()
parser.add_argument("-data", "--train_data", type=str, help="Training data")
parser.add_argument(
    "-rate",
    "--sampling_ratio",
    type=float,
    default=0.3,
    help="The spliting rate of the testing data.",
)
parser.add_argument(
    "-n_cluster", "--n_cluster", type=int, default=10, help="Number of clusters"
)
parser.add_argument(
    "-path",
    "--model_path",
    type=str,
    default=None,
    help="The path that saves the trained model",
)
parser.add_argument(
    "-cluster_result_format",
    "--cluster_result_format",
    type=str,
    default="csv",
    help="Save the result in csv format or txt format",
)
parser.add_argument(
    "-load_model_path",
    "--load_model_path",
    type=str,
    default=None,
    help="The path that loads the trained model",
    )
args = parser.parse_args()


def __check_args():
    if args.train_data is not None and args.train_data[-3:] not in [
        "npy",
        "csv",
        "txt",
    ]:
        raise ValueError("The training data must be in .npy, .csv or .txt format.")
    if args.sampling_ratio is not None and (
        args.sampling_ratio <= 0 or args.sampling_ratio > 1
    ):
        raise ValueError(
            "The sampling_ratio must be floating point and between 0 and 1."
        )
    if args.n_cluster is not None and args.n_cluster <= 0:
        raise ValueError("n_cluster must be integer and greater than 0.")
    if args.cluster_result_format is not None and args.cluster_result_format not in [
        "csv",
        "txt",
    ]:
        raise ValueError("The saving format must be .csv or .txt")


def __load_data():
    if args.train_data[-3:] == "npy":
        x = np.load(args.train_data)
    elif args.train_data[-3:] == "csv":
        df = pd.read_csv(args.train_data, header=None)
        x = df.to_numpy()
    elif args.train_data[-3:] == "txt":
        x = np.loadtxt(args.train_data, dtype=int)

    return x


def main(argv):
    __check_args()

    x = __load_data()

    if args.load_model_path is not None:
        psc = PSC()
        psc.load_model(args.load_model_path)
        cluster_idx = psc.predict(x)
    else:
        cluster_method = KMeans(
            n_clusters=args.n_cluster,
            init="k-means++",
            n_init=1,
            max_iter=100,
            algorithm="elkan",
        )
        psc = PSC(
            clustering_method=cluster_method,
            sampling_ratio=args.sampling_ratio,
            n_neighbor=args.n_cluster,
        )
        cluster_idx = psc.fit_predict(x)

    if args.model_path is not None:
        psc.save_model(args.model_path)

    if args.cluster_result_format == "csv":
        df = pd.DataFrame(cluster_idx)
        df.to_csv(
            args.train_data[:-4] + "_cluster_result.csv", index=False, header=False
        )

    elif args.result_saving_foramt == "txt":
        f = open(args.train_data[:-4] + "_cluster_result.txt", "w")
        np.set_printoptions(threshold=sys.maxsize)
        f.write(str(cluster_idx) + " ")
        f.close()

    print("Finished")


if __name__ == "__main__":
    main(sys.argv)



import numpy as np
import sys
import argparse
import pandas as pd
from ParametricSpectralClustering import PSC


parser = argparse.ArgumentParser()
parser.add_argument("-data", "--train_data", type=str, help="Training data")
parser.add_argument(
    "-rate",
    "--test_splitting_rate",
    type=float,
    default=0.3,
    help="The splitting rate of the training data",
)
parser.add_argument(
    "-path", "--model_path", type=str, help="A string that contains the file name"
)

args = parser.parse_args()


def __check_args():
    if args.train_data is not None and args.train_data[-3:] not in [
        "npy",
        "csv",
        "txt",
    ]:
        raise ValueError("The training data must be in .npy, .csv or .txt format.")
    if args.test_splitting_rate is not None and (
        args.test_splitting_rate <= 0 or args.test_splitting_rate > 1
    ):
        raise ValueError(
            "Test_splitting_rate must be floating point and between 0 and 1."
        )


def __load_data():
    if args.train_data[-3:] == "npy":
        # load from npy
        x = np.load(args.train_data)
    elif args.train_data[-3:] == "csv":
        # load from csv
        df = pd.read_csv(args.train_data, header=None)
        x = df.to_numpy()
    elif args.train_data[-3:] == "txt":
        # load from txt
        x = np.loadtxt(args.train_data, dtype=int)

    return x


def main(argv):
    __check_args()

    x = __load_data()

    psc = PSC(test_splitting_rate=args.test_splitting_rate)

    psc.training_psc_model(x)

    psc.save_model(args.model_path)

    print("Finished training.")


if __name__ == "__main__":
    main(sys.argv)



from setuptools import setup, find_packages

classifiers = [
    "Development Status :: 3 - Alpha",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 3.10",
    "Topic :: Scientific/Engineering :: Artificial Intelligence",
    "Operating System :: OS Independent",
]

requirements = [
    "scipy>=1.13.0",
    "torch>=2.2.2",
    "torchaudio>=2.2.2",
    "torchvision>=0.17.2",
    "scikit-learn>=1.4.2",
    "pandas>=2.2.2",
    "numpy==1.26.4",
    "matplotlib>=3.8.4",
]

scripts = [
    "bin/clustering.py",
    "bin/train_psc.py",
    "bin/run.py",
]

keywords = [
    "Spectral Clustering",
    "Incremental Clustering",
    "Online Clustering",
    "Non-linear clustering",
]

setup(
    name="ParametricSpectralClustering",
    version="0.0.5",
    description="A library for users to use parametric spectral clustering",
    long_description=open("README.md").read() + "\n\n" + open("CHANGELOG.txt").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests*"]),
    classifiers=classifiers,
    keywords=keywords,
    url="",
    author="Ivy Chang, Hsin Ju Tai",
    author_email="ivy900403@gmail.com, luludai020127@gmail.com",
    license="MIT",
    install_requires=requirements,
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    scripts=scripts,
)



import unittest
from ParametricSpectralClustering import PSC, Four_layer_FNN
from sklearn.cluster import KMeans
from sklearn.datasets import load_digits

digits = load_digits()
x = digits.data / 16
model = Four_layer_FNN(64, 128, 256, 64, 10)
clustering_method = KMeans(
    n_clusters=10, init="k-means++", n_init=1, max_iter=100, algorithm="elkan"
)


class NullClusteringMethod:
    def __init__(self, x) -> None:
        self.nullString = ""
        self.x = x

    def someRandomFunction(self):
        self.nullString = "some random function"


null_clustering_method = NullClusteringMethod(x)


class ErrorHandling(unittest.TestCase):
    def test_raise_fit_predict_AttributeError(self):
        psc = PSC(
            model=model, clustering_method=null_clustering_method, batch_size_data=1797
        )
        with self.assertRaises(AttributeError):
            psc.fit_predict(x)

    def test_raise_predict_AttributeError(self):
        psc = PSC(
            model=model, clustering_method=null_clustering_method, batch_size_data=1797
        )
        with self.assertRaises(AttributeError):
            psc.predict(x)

    def test_raise_load_model_FileNotFoundError(self):
        psc = PSC(
            model=model, clustering_method=clustering_method, batch_size_data=1797
        )
        with self.assertRaises(FileNotFoundError):
            psc.load_model("omg")


if __name__ == "__main__":
    unittest.main()
